import contextlib
import os
import tempfile
import unittest
from argparse import Namespace
from io import StringIO
import responses
from requests import Response
from a28 import api, utils
from a28.api import API, ApiError
from a28.config import ConfigFile


class TestApiMethods(unittest.TestCase):

    def setUp(self) -> None:
        self.test_dir = tempfile.TemporaryDirectory()
        utils.CONFIG_PATH = self.test_dir.name
        utils.CONFIG = os.path.join(utils.CONFIG_PATH, 'config.json')

        self.responses = responses.RequestsMock()
        self.responses.start()

        self.temp_stdout = StringIO()

        self.addCleanup(self.responses.stop)
        self.addCleanup(self.responses.reset)

    def tearDown(self) -> None:
        self.test_dir.cleanup()

    def test_initialize_populate_globals_from_config(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1/"
                    }
                }
            },
            status=200
        )

        api_client = API('international')

        self.assertEqual('https://api.example.com/v1/', api_client.api_endpoint)
        self.assertEqual('international', api_client.region)

    def test_initialize_throws_if_config_is_invalid(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "boo": {
                    "api": {
                        "endpoint": "https://api.example.com/v1/"
                    }
                }
            },
            status=200
        )

        with self.assertRaises(KeyError):
            with contextlib.redirect_stdout(self.temp_stdout):
                API('international')

        self.assertEqual("invalid endpoint", self.temp_stdout.getvalue().strip())

    def test_check_token_with_valid_response(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1/"
                    }
                }
            },
            status=200
        )
        api_client = API('international')

        resp = Response()

        resp._content = b'{"statusCode":200,"status":"SUCCESS","token":"supersecrettoken11"}'
        resp.status_code = 200

        self.assertEqual({}, ConfigFile.load())

        api_client._check_token(resp)

        self.assertEqual({'international': {'token': 'supersecrettoken11'}}, ConfigFile.load())

    def test_check_token_with_non_json_response(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1/"
                    }
                }
            },
            status=200
        )
        api_client = API('international')
        resp = Response()

        resp._content = b'what'
        resp.status_code = 200

        self.assertEqual({}, ConfigFile.load())

        api_client._check_token(resp)

        self.assertEqual({}, ConfigFile.load())

    def test_check_token_with_valid_response_but_no_token(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1/"
                    }
                }
            },
            status=200
        )
        api_client = API('international')
        resp = Response()

        resp._content = b'{"statusCode":200,"status":"SUCCESS"}'
        resp.status_code = 200

        self.assertEqual({}, ConfigFile.load())

        api_client._check_token(resp)

        self.assertEqual({}, ConfigFile.load())

    def test_endpoint_with_valid_response(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "uk": {
                    "api": {
                        "endpoint": "https://api.example.com/v1/"
                    }
                },
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1/"
                    }
                },
                "france": {
                    "api": {
                        "endpoint": "https://api.example.com/v1/"
                    }
                }
            },
            status=200
        )

        with contextlib.redirect_stdout(self.temp_stdout):
            api.endpoints(Namespace())

        self.assertEqual("uk\ninternational\nfrance\ndev", self.temp_stdout.getvalue().strip())

    def test_status_with_correct_response(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1"
                    }
                }
            },
            status=200
        )

        self.responses.add(
            responses.GET,
            'https://api.example.com/v1/status',
            json={"status": "SUCCESS", "statusCode": 200, "message": "im a message"},
            status=200
        )

        with contextlib.redirect_stdout(self.temp_stdout):
            api.status(Namespace(endpoint="international"))

        self.assertEqual("im a message", self.temp_stdout.getvalue().strip())

    def test_status_with_incorrect_response(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1"
                    }
                }
            },
            status=200
        )

        self.responses.add(
            responses.GET,
            'https://api.example.com/v1/status',
            json={"status": "SUCCESS", "statusCode": 200},
            status=200
        )

        with self.assertRaises(KeyError):
            with contextlib.redirect_stdout(self.temp_stdout):
                api.status(Namespace(endpoint="international"))

        self.assertEqual("unable to connect to endpoint", self.temp_stdout.getvalue().strip())



import subprocess


def capture(command):
    proc = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    out, err = proc.communicate()
    return out, err, proc.returncode



"""All the utils functions."""
import json
import os
import re
import sys
from typing import Any, Dict


# set the platform currently being used and the major and minor Python version
PLATFORM = sys.platform
PYTHON_MAJOR = sys.version_info.major
PYTHON_MINOR = sys.version_info.minor

# set the platform type (use startswith to handle old versions with ver nums)
POSIX = os.name == 'posix'
WINDOWS = os.name == 'nt'
LINUX = sys.platform.startswith('linux')
MACOS = sys.platform.startswith('darwin')
OSX = MACOS  # deprecated alias (thanks David F for the reminder)
FREEBSD = sys.platform.startswith('freebsd')
OPENBSD = sys.platform.startswith('openbsd')
NETBSD = sys.platform.startswith('netbsd')
BSD = FREEBSD or OPENBSD or NETBSD
SUNOS = sys.platform.startswith(('sunos', 'solaris'))
AIX = sys.platform.startswith('aix')

# get the users home path for storing files
if POSIX:
    HOMEDIR = os.path.expanduser('~')
elif WINDOWS:
    HOMEDIR = os.environ['APPDATA']
else:
    # unsupported operating system
    HOMEDIR = os.path.realpath(os.sep)

# set the storage constant
if MACOS:
    APP_SUP = os.path.join(HOMEDIR, 'Library', 'Application Support')
    STORAGE = os.path.join(APP_SUP, 'Area28', 'core')
elif WINDOWS:
    STORAGE = os.path.join(HOMEDIR, 'Area28', 'core')
elif POSIX:
    STORAGE = os.path.join(HOMEDIR, '.config', 'area28', 'core')
else:
    STORAGE = os.path.join(HOMEDIR, '.area28', 'core')

# set the devkit configuration location
CONFIG_PATH = os.path.realpath(os.path.join(STORAGE, os.pardir, 'devkit'))
CONFIG = os.path.join(CONFIG_PATH, 'config.json')


def message(msg='', flush=False, sep=' ', end='\n'):
    """Display a message to the user."""
    print(msg, sep=sep, end=end, flush=flush)


def confirm(msg='confirm?'):
    """Ask the user to confirm their action.

    Ask the user to confirm that ALL files and directories should be deleted
    from the directory specified by the storage variable. The STORAGE variable
    changes depending on the operating system used.
    """
    return bool(
        input(msg) in ['y', 'Y', 'yes', 'ok', 'sure', 'alrighty', 'of course', 'yup', 'oui']
    )


def valid_string(value, min_length=3, max_length=64):
    """Validate string input.

    Check if a string contains only alphanumeric, hyphen, and underscore
    characters and limited to 64 characters.
    """
    check = rf'^[\w_-]{{{min_length},{max_length}}}$'
    return re.match(check, value) is not None


def get_devkit_setting_path() -> str:
    """Get command line setting path."""
    return os.path.join(CONFIG_PATH, 'devkit_settings.json')


def get_devkit_settings() -> Dict[str, Any]:
    """Get command line setting."""
    devkit_settings = {}

    if not os.path.exists(get_devkit_setting_path()):
        return devkit_settings

    with open(get_devkit_setting_path(), 'r') as json_file:
        devkit_settings = json.load(json_file)

    return devkit_settings


def save_devkit_settings(settings: Dict[str, Any]) -> None:
    """Save command line setting."""
    # create the default config path if it doesn't exist
    os.makedirs(CONFIG_PATH, exist_ok=True)

    settings_content = json.dumps(settings, indent=4)
    with open(get_devkit_setting_path(), 'w') as settings_file:
        settings_file.write(settings_content)


def get_current_config_dir() -> str:
    """Get current config directory according to setting.

    return a string of the current config
    """
    settings = get_devkit_settings()
    current_stage = settings.get('current_stage', '')

    if current_stage in ['dev', 'staging']:
        return os.path.join(CONFIG_PATH, current_stage)
    # else current stage is prod

    return CONFIG_PATH


def get_current_config_path() -> str:
    """Get current config path according to setting."""
    return os.path.join(get_current_config_dir(), 'config.json')



import contextlib
from io import StringIO
from a28 import __version__
from a28.cli import main


def test_a28_no_param():
    temp_stdout = StringIO()

    try:
        with contextlib.redirect_stderr(temp_stdout):
            main()
    except SystemExit as err:
        assert err.code == 2

    assert temp_stdout.getvalue().strip().splitlines()[
               0] == 'usage: a28 [-h] [-v] {api,package,pkg,system,sys,account} ...'


def test_a28_version_param():
    temp_stdout = StringIO()

    try:
        with contextlib.redirect_stdout(temp_stdout):
            main(['-v'])
    except SystemExit as err:
        assert err.code == 0

    assert temp_stdout.getvalue().strip() == f'a28 version {__version__}'


def test_a28_system_exit_on_exception():
    """command line call that fail would trigger a system exit with code other than 0"""
    try:
        main(['account', 'authenticate', '-u', 'joe@example.com', '-p', '234'])
    except SystemExit as err:
        assert err.code != 0
    else:
        assert False



import contextlib
import os
import tempfile
import unittest
from argparse import Namespace
from io import StringIO
from a28.build import build


class TestAccountMethods(unittest.TestCase):
    def setUp(self) -> None:
        self.test_dir = tempfile.TemporaryDirectory()
        self.temp_stdout = StringIO()
        self.test_package_path = f'{os.path.dirname(__file__)}/fixtures/fake-pkg'

    def tearDown(self) -> None:
        self.test_dir.cleanup()

    def test_successful_build(self):
        with contextlib.redirect_stdout(self.temp_stdout):
            build(Namespace(src=self.test_package_path, dest=self.test_dir.name))

        self.assertEqual(
            f'{self.test_dir.name}/e3c02b90-3513-4742-81c5-fefa3abee637-0.1.0.a28',
            self.temp_stdout.getvalue().strip()
        )

    def test_successful_build_new_folder(self):
        with contextlib.redirect_stdout(self.temp_stdout):
            build(Namespace(src=self.test_package_path, dest=f'{self.test_dir.name}/newfolder'))

        self.assertEqual(
            f'{self.test_dir.name}/newfolder/e3c02b90-3513-4742-81c5-fefa3abee637-0.1.0.a28',
            self.temp_stdout.getvalue().strip()
        )



import contextlib
import os
import tempfile
import unittest
from argparse import Namespace
from io import StringIO
import responses
from responses import matchers
from a28 import utils, config
from a28.config import ConfigFile
from a28.publish import publish
from a28.api import ApiError

class TestPublishMethods(unittest.TestCase):
    def setUp(self) -> None:
        self.test_dir = tempfile.TemporaryDirectory()
        utils.CONFIG_PATH = self.test_dir.name
        utils.CONFIG = os.path.join(utils.CONFIG_PATH, 'config.json')

        self.responses = responses.RequestsMock()
        self.responses.start()

        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1"
                    }
                }
            },
            status=200
        )

        with ConfigFile() as data:
            data['international'] = {
                'email': 'joe@example.com',
                'token': 'supersecrettoken11'
            }

        self.temp_stdout = StringIO()

        self.test_file_path = \
            f'{os.path.dirname(__file__)}/fixtures/814a3feb-e9b2-4a5f-9088-4b108b47b0e9-6.6.6.a28'

        self.addCleanup(self.responses.stop)
        self.addCleanup(self.responses.reset)

    def tearDown(self) -> None:
        self.test_dir.cleanup()

    def test_successful_version_publish(self):
        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/asset/generate',
            json={
                "statusCode": 200,
                "status": "SUCCESS",
                "data": {
                    "url": 'https://fakes3.example.com/bucket/uploadme',
                    "name": 'package-6.6.6.a28'
                }
            },
            status=200,
            match=[matchers.json_params_matcher({
                "extension": "a28"
            })]
        )

        self.responses.add(
            responses.PUT,
            'https://fakes3.example.com/bucket/uploadme',
            status=200
        )

        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/package/814a3feb-e9b2-4a5f-9088-4b108b47b0e9/version',
            json={
                "statusCode": 201,
                "status": "SUCCESS"
            },
            status=201,
            match=[matchers.json_params_matcher({
                'package': 'package-6.6.6.a28',
                'version': '6.6.6'
            })]
        )

        with contextlib.redirect_stdout(self.temp_stdout):
            publish(Namespace(
                endpoint='international',
                pkg=self.test_file_path,
            ))

        self.assertEqual("Uploading...\nDone!", self.temp_stdout.getvalue().strip())

    def test_version_publish_not_auth(self):
        os.remove(utils.get_current_config_path())

        with self.assertRaises(config.ConfigError):
            with contextlib.redirect_stdout(self.temp_stdout):
                publish(Namespace(
                    endpoint='international',
                    pkg=self.test_file_path,
                ))

        self.assertEqual(
            "Uploading\nFailed: please authenticate",
            self.temp_stdout.getvalue().strip(),
        )

    def test_version_publish_fail_to_find_file(self):

        with self.assertRaises(FileNotFoundError):
            with contextlib.redirect_stdout(self.temp_stdout):
                publish(Namespace(
                    endpoint='international',
                    pkg='./fixtures/dontexist.a28',
                ))

        self.assertEqual(
            "Uploading\nFailed: [Errno 2] No such file or directory: './fixtures/dontexist.a28'",
            self.temp_stdout.getvalue().strip(),
        )

    def test_version_publish_fail_generate_upload_url(self):
        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/asset/generate',
            json={
                "statusCode": 402,
                "status": "ERROR",
                "message": "I don't like you"
            },
            status=402
        )

        with self.assertRaises(ApiError):
            with contextlib.redirect_stdout(self.temp_stdout):
                publish(Namespace(
                    endpoint='international',
                    pkg=self.test_file_path,
                ))

        self.assertEqual(
            "Uploading.\nFailed: I don't like you",
            self.temp_stdout.getvalue().strip(),
        )

    def test_version_publish_fail_upload_to_s3(self):
        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/asset/generate',
            json={
                "statusCode": 200,
                "status": "SUCCESS",
                "data": {
                    "url": 'https://fakes3.example.com/bucket/uploadme',
                    "name": 'package-6.6.6.a28'
                }
            },
            status=200
        )

        self.responses.add(
            responses.PUT,
            'https://fakes3.example.com/bucket/uploadme',
            status=500
        )

        with self.assertRaises(ApiError):
            with contextlib.redirect_stdout(self.temp_stdout):
                publish(Namespace(
                    endpoint='international',
                    pkg=self.test_file_path,
                ))

        self.assertEqual(
            "Uploading.\nFailed: 500",
            self.temp_stdout.getvalue().strip(),
        )

    def test_version_publish_fail_create_version(self):
        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/asset/generate',
            json={
                "statusCode": 200,
                "status": "SUCCESS",
                "data": {
                    "url": 'https://fakes3.example.com/bucket/uploadme',
                    "name": 'package-6.6.6.a28'
                }
            },
            status=200
        )

        self.responses.add(
            responses.PUT,
            'https://fakes3.example.com/bucket/uploadme',
            status=200
        )

        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/package/814a3feb-e9b2-4a5f-9088-4b108b47b0e9/version',
            json={
                "statusCode": 409,
                "status": "ERROR",
                "message": "version already exists"
            },
            status=409
        )

        with self.assertRaises(ApiError):
            with contextlib.redirect_stdout(self.temp_stdout):
                publish(Namespace(
                    endpoint='international',
                    pkg=self.test_file_path,
                ))

        self.assertEqual(
            "Uploading..\nFailed: version already exists",
            self.temp_stdout.getvalue().strip(),
        )



import contextlib
import os
import tempfile
import unittest
from argparse import Namespace
from importlib import reload
from io import StringIO
import responses
from a28 import account, config, utils
from a28.config import ConfigFile
from a28.api import ApiError

class TestAccountMethods(unittest.TestCase):
    def setUp(self) -> None:
        self.test_dir = tempfile.TemporaryDirectory()
        utils.CONFIG_PATH = self.test_dir.name
        utils.CONFIG = os.path.join(utils.CONFIG_PATH, 'config.json')

        reload(config)

        self.responses = responses.RequestsMock()
        self.responses.start()

        self.temp_stdout = StringIO()

        self.addCleanup(self.responses.stop)
        self.addCleanup(self.responses.reset)

    def tearDown(self) -> None:
        self.test_dir.cleanup()

    def test_authenticate_with_valid_response(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1"
                    }
                }
            },
            status=200
        )

        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/account/signin',
            json={
                "statusCode": 200,
                "status": "SUCCESS",
                "token": "supersecrettoken11"
            },
            status=200
        )
        self.assertEqual({}, ConfigFile.load())

        with contextlib.redirect_stdout(self.temp_stdout):
            account.authenticate(region="international", email="joe@example.com", password="123")

        self.assertEqual("authentication successful", self.temp_stdout.getvalue().strip())
        self.assertEqual(
            {
                'international': {
                    'email': 'joe@example.com',
                    'token': 'supersecrettoken11'
                }
            },
            ConfigFile.load()
        )

    def test_authenticate_with_param_password(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1"
                    }
                }
            },
            status=200
        )

        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/account/signin',
            json={
                "statusCode": 200,
                "status": "SUCCESS",
                "token": "supersecrettoken11"
            },
            status=200
        )
        self.assertEqual({}, ConfigFile.load())

        with contextlib.redirect_stdout(self.temp_stdout):
            account.authenticate_interactive(
                Namespace(endpoint="international", email="joe@example.com", password="123")
            )

        self.assertEqual(
            '''Warning: Using a password on the command line interface can be insecure.
authentication successful''',
            self.temp_stdout.getvalue().strip()
        )
        self.assertEqual(
            {
                'international': {
                    'email': 'joe@example.com',
                    'token': 'supersecrettoken11'
                }
            },
            ConfigFile.load()
        )

    def test_authenticate_with_unauthorized_response(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1"
                    }
                }
            },
            status=200
        )

        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/account/signin',
            json={
                "statusCode": 402,
                "status": "ERROR",
                "message": "you can't"
            },
            status=402
        )
        self.assertEqual({}, ConfigFile.load())

        with self.assertRaises(ApiError):
            with contextlib.redirect_stdout(self.temp_stdout):
                account.authenticate(region="international", email="joe@example.com", password="123")

        self.assertEqual("error authenticating: you can't", self.temp_stdout.getvalue().strip())
        self.assertEqual({}, ConfigFile.load())

    def test_authenticate_with_failed_response(self):
        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1"
                    }
                }
            },
            status=200
        )

        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/account/signin',
            json={},
            status=500
        )
        self.assertEqual({}, ConfigFile.load())

        with self.assertRaises(ApiError):
            with contextlib.redirect_stdout(self.temp_stdout):
                account.authenticate(region="international", email="joe@example.com", password="123")

        self.assertEqual("error authenticating: 500", self.temp_stdout.getvalue().strip())
        self.assertEqual({}, ConfigFile.load())



"""Manage actions relating to the system running area28.

The actions provided by the system module adds the ability to manage the state
of the computer system running Area28 allowing for developers to maintain the
configuration and paths of the system.
"""
import os
import shutil
from argparse import Namespace, _SubParsersAction
from a28 import utils


def cli_options(mainparser: _SubParsersAction) -> None:
    """System subcommand arguments.

    Define the arguments that the system subcommand can accept. The system
    subcommand supports the following commands:

    clean: Clean all files from the current system.
    exists: Return if the system currently contains a configuration.
    path: Return the current system configuration path.
    """
    parser = mainparser.add_parser(
        'system',
        aliases=['sys'],
        help='system actions'
    )
    subparser = parser.add_subparsers(
        dest='system',
        required=True,
        help='system',
    )
    parser.add_argument(
        '-p',
        '--path',
        default=utils.get_current_config_dir(),
        help='configuration path.',
    )
    parser_exist = subparser.add_parser(
        'exists',
        help='check if the system configuration exists.',
    )
    parser_exist.set_defaults(func=exists)
    parser_clean = subparser.add_parser(
        'clean',
        help='clean (delete) the configuration permanently.',
    )
    parser_clean.set_defaults(func=clean)
    parser_clean.add_argument(
        '-f',
        '--force',
        action='store_true',
        help='force the configuration to be removed bypassing confirmation',
    )
    parser_path = subparser.add_parser(
        'path',
        help='return the location of the configuration path.',
    )
    parser_path.set_defaults(func=path)
    parser_path.add_argument(
        '-m',
        '--minimal',
        action='store_true',
        help='return a minimal version of the path to use in scripts',
    )
    parser_stage = subparser.add_parser(
        'stage',
        help='set the stage of current configuration'
    )
    parser_stage.set_defaults(func=stage)
    parser_stage.add_argument(
        '-s',
        '--stage',
        dest='stage',
        required=True,
        default='prod',
        choices=['prod', 'staging', 'dev'],
        help='switch staging mode'
    )


def clean(args: Namespace) -> None:
    """Clean all the configuration files relating to Area28.

    Using the shutil utilities, delete all the files provided in the STORAGE
    directory. If an error occurs, catch the exception and print it out to the
    STDOUT.

    Args:
        args (Namespace): The list of provided arguments.

    Returns:
        None: Does not return any value.
    """
    message = f'Delete ALL configuration data from "{args.path}"? y/n?'
    if not os.path.isdir(args.path):
        utils.message(f'No configuration to clean at "{args.path}".')
        raise FileNotFoundError

    elif not args.force and not utils.confirm(message):
        utils.message(f'Not deleting {args.path}')
        return

    try:
        shutil.rmtree(args.path)
        utils.message(f'Deleted all files and directories in {args.path}')
    except OSError as e:
        utils.message(f'Error: {args.path} : {e.strerror}')
        raise


def exists(args: Namespace) -> None:
    """Check if the configuration directory exists.

    Using the path provided in the arguments, check if the configuration path
    exists. If no path is provided, the default path defined in the system
    utility is used.

    Args:
        args (Namespace): The list of provided arguments.

    Returns:
        None: Does not return any value.
    """
    if os.path.isdir(args.path):
        utils.message(f'Configuration exists at "{args.path}".')
    else:
        utils.message(f'No configuration exists at "{args.path}".')


def path(args: Namespace) -> None:
    """Print the path of the system configuration.

    Print the path currently used to store the system configuration files. If
    the path option is provided in the ArgParse Namespace, the provided path
    will be returned.

    If the minimal ArgParse namespace is provided and set to boolean True, the
    output will be reduced to only the path.

    It does NOT change the configuration path

    Args:
        args (Namespace): The list of provided arguments.

    Returns:
        None: Does not return any value.
    """
    if args.minimal:
        utils.message(f'{args.path}')
    else:
        utils.message(f'Configuration path set to "{args.path}".')


def stage(args: Namespace) -> None:
    """Set the stage of the environment configuration.

    record the current stage at command line setting file in default path
    Get current config dir/path in utils.py would return the value accordingly

    Args:
        args (Namespace): The list of provided arguments.

    Returns:
        None: Does not return any value.
    """
    devkit_settings = utils.get_devkit_settings()
    devkit_settings['current_stage'] = args.stage
    utils.save_devkit_settings(devkit_settings)
    utils.message(f'Current stage set to {args.stage}')



# vim: encoding=utf-8 ts=4 et sts=4 sw=4 tw=79 fileformat=unix nu wm=2
import argparse
import utils
from a28.package import generate_jsondata


def test_a28_package_no_param():
    command = ["a28", "package"]
    out, err, exitcode = utils.capture(command)
    assert exitcode == 2
    assert out == b""
    message = b"usage: a28 package [-h] {init,meta,build,install,publish} ..."
    assert err[0 : len(message)] == message


def test_a28_init_no_param():
    command = ["a28", "package", "init"]
    out, err, exitcode = utils.capture(command)
    assert exitcode == 2
    assert out == b""
    message = b"usage: a28 package init [-h] -s SCOPE -n NAME -t"
    assert err[0 : len(message)] == message


def test_a28_build_no_param():
    command = ["a28", "package", "build"]
    out, err, exitcode = utils.capture(command)
    assert exitcode == 2
    assert out == b""
    message = b"usage: a28 package build [-h] --src SRC [--dest DEST]"
    assert err[0 : len(message)] == message


def test_a28_install_no_param():
    command = ["a28", "package", "install"]
    out, err, exitcode = utils.capture(command)
    assert exitcode == 2
    assert out == b""
    message = b"usage: a28 package install [-h] --pkg PKG"
    assert err[0 : len(message)] == message


def test_generate_jsondata_without_args():
    result = generate_jsondata(
        scope="foo",
        name="bar",
        identifier="1234",
        schema="234",
        args=argparse.Namespace(bin="", script=""),
    )

    assert result == {
        "description": "234 package created using the A28 command by foo.",
        "identifier": "1234",
        "name": "@foo/bar",
        "version": "0.1.0",
    }


def test_generate_jsondata_with_bin():
    result = generate_jsondata(
        scope="foo",
        name="bar",
        identifier="1234",
        schema="234",
        args=argparse.Namespace(bin="in the bin", script=""),
    )

    assert result == {
        "description": "234 package created using the A28 command by foo.",
        "identifier": "1234",
        "name": "@foo/bar",
        "version": "0.1.0",
        "bin": {},
    }


def test_generate_jsondata_with_bin_and_script():
    result = generate_jsondata(
        scope="foo",
        name="bar",
        identifier="1234",
        schema="234",
        args=argparse.Namespace(bin="in the bin", script="the script"),
    )

    assert result == {
        "description": "234 package created using the A28 command by foo.",
        "identifier": "1234",
        "name": "@foo/bar",
        "version": "0.1.0",
        "bin": {},
        "scripts": {},
    }



"""Build packages to be submitted to Area28."""
import argparse
import sys
from a28 import __version__, account, api, package, system


def main(args=None):
    """Command line entry point."""
    args = args or sys.argv[1:]
    parser = argparse.ArgumentParser(
        'a28',
        description='Area28 development kit'
    )
    app = '%(prog)s version ' + __version__
    parser.add_argument('-v', '--version', action='version', version=app)
    sub = parser.add_subparsers(dest='action', required=True, help='actions')

    try:
        api.cli_options(sub)
        package.cli_options(sub)
        system.cli_options(sub)
        account.cli_options(sub)
        args = parser.parse_args(args)
        args.func(args)

    except Exception:
        sys.exit(1)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['a28']

package_data = \
{'': ['*']}

install_requires = \
['certifi>=2021.10.8,<2022.0.0',
 'coloredlogs[color]>=14.0',
 'requests>=2.23.0,<3.0.0']

entry_points = \
{'console_scripts': ['a28 = a28.cli:main']}

setup_kwargs = {
    'name': 'a28',
    'version': '1.1.0',
    'description': 'A set of resources and tools to help developers create packages for the Area28 application.',
    'long_description': '# Developer Toolkit\n\nDeveloper Toolkit providing developers with the tools and documentation necessary to build packages for the Area28\nApplication.\n\n## Python version\n\nArea28 follows the [VFX Reference Platform](https://vfxplatform.com/) which restricts Python to 3.7.x currently.\n\n## Interfaces\n\n- IPlugin\n- IApiExtension\n- IApplicationExtension\n- IChatExtension\n- IEventExtension\n- IInteractionExtension\n- ILoggerExtension\n- IMetadataExtension\n- IPreferencesExtension\n- IRealtimeExtension\n- IUiExtension\n- IUnitsExtension\n\n## Plugins\n\nPlugins are decorators that can be used to manipulate the payload before being processed or before getting returned.\n\n## Extensions\n\nExtension are used to add additional functionality to the Area28 application. Extensions are broken up into multiple\ntypes, defined within the Interfaces list.\n\n## Packaging\n\nEach package has a unique identifier associated with it and is compressed into a .a28 file. Please look at the a28\ndevelopment kit for details.\n\n### Package structure\n\n```sh\n@{provider}\n|-- {package}\n    |-- extensions\n    |   |-- {extensions[]}.py\n    |-- scripts\n    |   |-- install.py\n    |   |-- postinstall.py\n    |   |-- preinstall.py\n    |   |-- uninstall.py\n    |-- plugin\n    |   |-- {application specific plugin}\n    |-- plugins\n    |   |-- {plugin[]}.py\n    |-- bin\n    |   |--{executable[]}.py\n    |-- package.json\n```\n\n### Package.json structure\n\n```json\n{\n    "name": "@area28/unity-application",\n    "version": "0.0.4",\n    "description": "Detect is running within Unity3D.",\n    "homepage": "https://area28.io",\n    "keywords": [\n        "area28",\n        "chat",\n        "lowercase",\n        "transform"\n    ],\n    "repository": {\n        "type": "git",\n        "url": "git+https://github.com/area28/area28.git",\n        "directory": "packages/unity-application"\n    },\n    "author": "Gary Stidston-Broadbent",\n    "license": "MIT",\n    "bugs": {\n        "url": "https://github.com/area28/area28/issues"\n    },\n    "bin": {\n        "myapp": "./bin/lowercase.py"\n    },\n    "os": [\n        "darwin",\n        "linux"\n    ],\n    "cpu": [\n        "x64",\n        "ia32",\n        "!mips"\n    ],\n    "scripts": {\n        "preinstall": "scripts/preinstall.py",\n        "install": "scripts/install.py",\n        "postinstall": "scripts/postinstall.py",\n        "uninstall": "scripts/uninstall.py"\n    }\n}\n```\n\n### Authenticate with A28\n\n- `a28 account authenticate -u my.email@example.com`\n\n### Initialize a new package\n\n- `a28 package init --scope my-company --name powerful-plugin --type app my/folder`\n\nThis will generate a `package.json` and the folder structure in the given folder (\'my/folder\')\n\n### Building a package\n\n- `a28 package build --src @area28/chat-logger --dest dist`\n\nThis will generate the `.a28` package in `dist/`\n\n### Installing a package locally\n\n- `a28 package install --pkg dist/00000000-0000-0000-0000-00000000-0.0.1.a28`\n\n### Publish a package\n\n- `a28 package publish --pkg dist/00000000-0000-0000-0000-00000000-0.0.1.a28`\n',
    'author': 'Area28 Technologies',
    'author_email': 'dev@area28.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://area28.io',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4',
}


setup(**setup_kwargs)



"""Build packages to be submitted to Area28."""
import logging
import coloredlogs


# current version
__version__ = '1.1.0'

# enable logging
logging.basicConfig()
log = logging.getLogger()
log.setLevel(logging.DEBUG)
coloredlogs.install(level='DEBUG', logger=log)



"""Create build of a package."""
import json
import os
import zipfile
from argparse import _SubParsersAction
from a28 import utils


def add_build_parser(parser: _SubParsersAction):
    """Create the CLI parser for 'build' command."""
    parser.set_defaults(func=build)
    parser.add_argument(
        '--src',
        required=True,
        help='package source directory',
    )
    parser.add_argument(
        '--dest',
        default='',
        help='destination directory',
    )


def build(args):
    """Build package."""
    src = args.src
    dest = args.dest
    meta = get_info(src)
    pkg = package(src, dest, meta)
    utils.message(pkg)


def package(src, dest, meta):
    """
    Build an a28 package from the provided src directory.

    The package will be saved to the dest directory. A package needs to be provided containing
    at least an identifier and a version number.
    """
    version = meta['version']
    identifier = meta['identifier']
    filename = '{}-{}.{}'.format(identifier, version, 'a28')
    filename = os.path.join(dest, filename)

    os.makedirs(dest, exist_ok=True)

    a28 = zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED)
    exclude = (['build', '.vscode', '.git', '.github', 'dist'])
    for root, dirs, files in os.walk(src):
        dirs[:] = [d for d in dirs if d not in exclude]
        for current in files:
            i_file = os.path.join(root, current)
            fl = os.path.relpath(
                os.path.join(root, current),
                os.path.join(src),
            )
            a28.write(i_file, fl)
    a28.close()
    return filename


def get_info(src, extracted=False):
    """
    Get information about a package.

    Using the package.json file located within the packages directory, fetch
    the information about the package including the version number and
    identifier.
    """
    if extracted:
        data = json.load(src)
    else:
        package_config = os.path.join(src, 'package.json')
        with open(package_config) as package_data:
            data = json.load(package_data)
    return data


def extract_meta(pkg):
    """Extract the meta information from a packaged package."""
    with zipfile.ZipFile(pkg) as zf:
        package_config = next(x for x in zf.namelist() if x.endswith('package.json'))
        with zf.open(package_config) as index:
            return get_info(index, True)



# vim: encoding=utf-8 ts=4 et sts=4 sw=4 tw=79 fileformat=unix nu wm=2
import pytest
from a28 import utils


def test_string_validation_short():
    assert utils.valid_string('ab', min_length=3) is False


def test_string_validation_long():
    assert utils.valid_string('abcdefghijklmnop', max_length=10) is False


def test_string_validation_invalid():
    assert utils.valid_string('!@£$%^&*(())__+') is False


def test_string_validation_valid():
    assert utils.valid_string('avalidword') is True


def test_confirm_nope(monkeypatch):
    # monkeypatch the "input" function, so that it returns "Mark".
    # This simulates the user entering "Mark" in the terminal:
    monkeypatch.setattr('builtins.input', lambda _: "nope")
    result = utils.confirm('test message')
    assert result is False


def test_confirm_y(monkeypatch):
    # monkeypatch the "input" function, so that it returns "Mark".
    # This simulates the user entering "Mark" in the terminal:
    monkeypatch.setattr('builtins.input', lambda _: "y")
    result = utils.confirm('test message')
    assert result is True


def test_confirm_yes(monkeypatch):
    # monkeypatch the "input" function, so that it returns "Mark".
    # This simulates the user entering "Mark" in the terminal:
    monkeypatch.setattr('builtins.input', lambda _: "yes")
    result = utils.confirm('test message')
    assert result is True


def test_confirm_ok(monkeypatch):
    # monkeypatch the "input" function, so that it returns "Mark".
    # This simulates the user entering "Mark" in the terminal:
    monkeypatch.setattr('builtins.input', lambda _: "ok")
    result = utils.confirm('test message')
    assert result is True


def test_confirm_sure(monkeypatch):
    # monkeypatch the "input" function, so that it returns "Mark".
    # This simulates the user entering "Mark" in the terminal:
    monkeypatch.setattr('builtins.input', lambda _: "sure")
    result = utils.confirm('test message')
    assert result is True


def test_confirm_alrighty(monkeypatch):
    # monkeypatch the "input" function, so that it returns "Mark".
    # This simulates the user entering "Mark" in the terminal:
    monkeypatch.setattr('builtins.input', lambda _: "alrighty")
    result = utils.confirm('test message')
    assert result is True


def test_confirm_ofcourse(monkeypatch):
    # monkeypatch the "input" function, so that it returns "Mark".
    # This simulates the user entering "Mark" in the terminal:
    monkeypatch.setattr('builtins.input', lambda _: "of course")
    result = utils.confirm('test message')
    assert result is True


def test_confirm_yup(monkeypatch):
    # monkeypatch the "input" function, so that it returns "Mark".
    # This simulates the user entering "Mark" in the terminal:
    monkeypatch.setattr('builtins.input', lambda _: "yup")
    result = utils.confirm('test message')
    assert result is True



import contextlib
import json
import os
import tempfile
import unittest
from argparse import Namespace
from importlib import reload
from io import StringIO
import responses
from a28 import config, package, utils
from a28.config import ConfigError, ConfigFile
from a28.api import ApiError


class TestPackageInitMethods(unittest.TestCase):
    def setUp(self) -> None:
        self.test_dir = tempfile.TemporaryDirectory()
        utils.CONFIG_PATH = self.test_dir.name
        utils.CONFIG = os.path.join(utils.CONFIG_PATH, 'config.json')

        reload(config)

        self.responses = responses.RequestsMock()
        self.responses.start()

        self.temp_stdout = StringIO()

        self.responses.add(
            responses.GET,
            'https://assets.a28.io/plugin/endpoints.json',
            json={
                "international": {
                    "api": {
                        "endpoint": "https://api.example.com/v1"
                    }
                }
            },
            status=200
        )

        with ConfigFile() as data:
            data['international'] = {
                'email': 'joe@example.com',
                'token': 'supersecrettoken11'
            }

        self.addCleanup(self.responses.stop)
        self.addCleanup(self.responses.reset)

    def tearDown(self) -> None:
        self.test_dir.cleanup()

    def test_successful_package_init(self):
        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/group/nicescopegroup/package',
            json={
                "statusCode": 201,
                "status": "SUCCESS",
                "data": {
                    "identifier": '1234567'
                }
            },
            status=201
        )

        with contextlib.redirect_stdout(self.temp_stdout):
            package.initialize(
                Namespace(
                    bin=None,
                    script=None,
                    identifier=None,
                    endpoint='international',
                    path=self.test_dir.name,
                    name='nicetest',
                    scope='nicescopegroup',
                    type='app',
                )
            )

        self.assertEqual("package created", self.temp_stdout.getvalue().strip())

        with open(f'{self.test_dir.name}/package.json') as json_file:
            data = json.load(json_file)
            self.assertEqual(
                {
                    'description': 'app package created using the A28 command by nicescopegroup.',
                    'identifier': '1234567',
                    'name': '@nicescopegroup/nicetest',
                    'version': '0.1.0'
                },
                data
            )

    def test_package_init_with_error(self):
        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/group/nicescopegroup/package',
            json={
                "statusCode": 500,
                "status": "ERROR",
                "message": "server crashed again"
            },
            status=500
        )

        with self.assertRaises(ApiError):
            with contextlib.redirect_stdout(self.temp_stdout):
                package.initialize(
                    Namespace(
                        bin=None,
                        script=None,
                        identifier=None,
                        endpoint='international',
                        path=self.test_dir.name,
                        name='nicetest',
                        scope='nicescopegroup',
                        type='app',
                    )
                )

        self.assertEqual(
            "unable to create package: server crashed again",
            self.temp_stdout.getvalue().strip()
        )
        self.assertEqual(False, os.path.isfile(f'{self.test_dir.name}/package.json'))

    def test_successful_package_init_on_new_folder(self):
        self.responses.add(
            responses.POST,
            'https://api.example.com/v1/group/nicescopegroup/package',
            json={
                "statusCode": 201,
                "status": "SUCCESS",
                "data": {
                    "identifier": 'imagreeatuuid'
                }
            },
            status=201
        )

        with contextlib.redirect_stdout(self.temp_stdout):
            package.initialize(
                Namespace(
                    bin=None,
                    script=None,
                    identifier=None,
                    endpoint='international',
                    path=f'{self.test_dir.name}/newfolder',
                    name='nicetest',
                    scope='nicescopegroup',
                    type='app',
                )
            )

        self.assertEqual("package created", self.temp_stdout.getvalue().strip())

        with open(f'{self.test_dir.name}/newfolder/package.json') as json_file:
            data = json.load(json_file)
            self.assertEqual(
                {
                    'description': 'app package created using the A28 command by nicescopegroup.',
                    'identifier': 'imagreeatuuid',
                    'name': '@nicescopegroup/nicetest',
                    'version': '0.1.0'
                },
                data
            )

    def test_package_init_with_no_config(self):
        os.remove(utils.get_current_config_path())

        with self.assertRaises(SystemExit) as err:
            with contextlib.redirect_stdout(self.temp_stdout):
                package.initialize(
                    Namespace(
                        bin=None,
                        script=None,
                        identifier=None,
                        endpoint='international',
                        path=self.test_dir.name,
                        name='nicetest',
                        scope='nicescopegroup',
                        type='app',
                    )
                )

        self.assertEqual(err.exception.code, 1)
        self.assertEqual(
            "authenticate before initializing",
            self.temp_stdout.getvalue().strip()
        )
        self.assertEqual(False, os.path.isfile(f'{self.test_dir.name}/package.json'))

    def test_successful_package_init_with_existing_uuid(self):
        self.responses.add(
            responses.GET,
            'https://api.example.com/v1/package/existinguuid11',
            json={
                "statusCode": 200,
                "status": "SUCCESS",
                "data": {
                    "identifier": 'existinguuid11'
                }
            },
            status=200
        )

        with contextlib.redirect_stdout(self.temp_stdout):
            package.initialize(
                Namespace(
                    bin=None,
                    script=None,
                    identifier='existinguuid11',
                    endpoint='international',
                    path=self.test_dir.name,
                    name='nicetest',
                    scope='nicescopegroup',
                    type='app',
                )
            )

        self.assertEqual("package created", self.temp_stdout.getvalue().strip())

        with open(f'{self.test_dir.name}/package.json') as json_file:
            data = json.load(json_file)
            self.assertEqual(
                {
                    'description': 'app package created using the A28 command by nicescopegroup.',
                    'identifier': 'existinguuid11',
                    'name': '@nicescopegroup/nicetest',
                    'version': '0.1.0'
                },
                data
            )

    def test_unsuccessful_package_init_with_existing_uuid(self):
        self.responses.add(
            responses.GET,
            'https://api.example.com/v1/package/existinguuid21',
            json={
                "statusCode": 404,
                "status": "ERROR",
                "message": "DOES NOT EXIST"
            },
            status=404
        )

        with self.assertRaises(ApiError):
            with contextlib.redirect_stdout(self.temp_stdout):
                package.initialize(
                    Namespace(
                        bin=None,
                        script=None,
                        identifier='existinguuid21',
                        endpoint='international',
                        path=self.test_dir.name,
                        name='nicetest',
                        scope='nicescopegroup',
                        type='app',
                    )
                )

        self.assertEqual(
            "unable to get package: DOES NOT EXIST",
            self.temp_stdout.getvalue().strip()
        )
        self.assertEqual(False, os.path.isfile(f'{self.test_dir.name}/package.json'))



# vim: encoding=utf-8 ts=4 et sts=4 sw=4 tw=79 fileformat=unix nu wm=2
import os.path
import subprocess
from pathlib import Path
import utils


def test_a28_system_no_param():
    command = ['a28', 'system']
    out, err, exitcode = utils.capture(command)
    assert exitcode == 2
    assert out == b''
    message = b'usage: a28 system [-h] [-p PATH] {exists,clean,path,stage} ...'
    assert err[0:len(message)] == message


def test_a28_exists_no_param_no_config(tmp_path: Path):
    d = tmp_path / 'configuration'
    command = ['a28', 'system', f'--path={d}', 'exists']
    out, _, exitcode = utils.capture(command)
    assert exitcode == 0
    message = b'No configuration exists at '
    message_len = len(message)
    assert out[0:message_len] == message


def test_a28_exists_no_param_config(tmp_path: Path):
    d = tmp_path / 'configuration'
    d.mkdir()
    command = ['a28', 'system', f'--path={d}', 'exists']
    out, _, exitcode = utils.capture(command)
    assert exitcode == 0
    message = b'Configuration exists at '
    message_len = len(message)
    assert out[0:message_len] == message


def test_a28_clean_no_param_no_config(tmp_path: Path):
    d = tmp_path / 'configuration_clean'
    command = ['a28', 'system', f'--path={d}', 'clean', '-f']
    out, _, exitcode = utils.capture(command)
    assert exitcode == 1
    message = b'No configuration to clean at '
    message_len = len(message)
    assert out[0:message_len] == message


def test_a28_clean_no_param_config(tmp_path: Path):
    d = tmp_path / 'configuration_clean'
    d.mkdir()
    command = ['a28', 'system', f'--path={d}', 'clean', '-f']
    out, _, exitcode = utils.capture(command)
    assert exitcode == 0
    message = b'Deleted all files and directories in '
    message_len = len(message)
    assert out[0:message_len] == message


def test_a28_path_minimal(tmp_path: Path):
    d = tmp_path / 'configuration_path'
    command = ['a28', 'system', f'--path={d}', 'path', '--minimal']
    out, _, exitcode = utils.capture(command)
    assert exitcode == 0
    message = d.__bytes__()
    assert out[0:len(message)] == message


def test_a28_path_no_minimal(tmp_path: Path):
    d = tmp_path / 'configuration_path'
    command = ['a28', 'system', f'--path={d}', 'path']
    out, _, exitcode = utils.capture(command)
    assert exitcode == 0
    message = b'Configuration path set to "' + d.__bytes__() + b'".\n'
    assert out[0:len(message)] == message

def test_a28_set_stage(tmp_path: Path):
    for stage in ['prod', 'staging', 'dev']:
        path = tmp_path / 'configuration_path'

        if stage != 'prod':
            path = os.path.join(path, stage)

        command = ['a28', 'system', 'stage', f'-s={stage}']
        out, _, exitcode = utils.capture(command)
        assert exitcode == 0
        message = f'Current stage set to {stage}\n'
        assert str(out[0:len(message)], 'utf-8') == message

        command = ['a28', 'system', f'--path={path}', 'path', '--minimal']
        out, _, exitcode = utils.capture(command)
        assert exitcode == 0
        message = f'{path}\n'
        assert str(out[0:len(message)], 'utf-8') == message
